@@ -29,10 +29,88 @@
 
 struct _MsgDriveService {
   MsgService parent_instance;
+
+  MsgDriveType type;
 };
 
 G_DEFINE_TYPE (MsgDriveService, msg_drive_service, MSG_TYPE_SERVICE);
 
+static gboolean
+is_valid_name (const char *name)
+{
+  g_auto (GStrv) split = NULL;
+  const char *invalid_names[] = {
+    ".lock",
+    "CON",
+    "PRN",
+    "AUX",
+    "NUL",
+    "COM0",
+    "COM1",
+    "COM2",
+    "COM3",
+    "COM4",
+    "COM5",
+    "COM6",
+    "COM7",
+    "COM8",
+    "COM9",
+    "LPT0",
+    "LPT1",
+    "LPT2",
+    "LPT3",
+    "LPT4",
+    "LPT5",
+    "LPT6",
+    "LPT7",
+    "LPT8",
+    "LPT9",
+    "_vti_",
+    "desktop.ini",
+    NULL
+  };
+  g_autofree char *lower_name = g_utf8_strdown (name, -1);
+
+  /* Check for invalid names */
+  for (int i = 0; invalid_names[i] != NULL; i++) {
+    g_autofree char *invalid_name = g_utf8_strdown (invalid_names[i], -1);
+
+    if (g_strcmp0 (lower_name, invalid_name) == 0)
+      return FALSE;
+  }
+
+  /* Leading or Trailing whitespaces are invalid */
+  if (g_str_has_prefix (name, " ") || g_str_has_suffix (name, " "))
+    return FALSE;
+
+  /* Must not start with ~$ */
+  if (g_str_has_prefix (name, "~$"))
+    return FALSE;
+
+  /* _vti_ cannot be part of the name */
+  if (strstr (name, "_vti_"))
+    return FALSE;
+
+  /* Not allowed chars: " * : < > ? / \ | */
+  if (strchr (name, '"') ||
+      strchr (name, '*') ||
+      strchr (name, ':') ||
+      strchr (name, '<') ||
+      strchr (name, '>') ||
+      strchr (name, '?') ||
+      strchr (name, '/') ||
+      strchr (name, '\\') ||
+      strchr (name, '|'))
+    return FALSE;
+
+  /* "forms" on root level for a library */
+  split = g_strsplit (name, "/", -1);
+  if (g_strv_length (split) == 2 && g_strcmp0 (split[1], "forms") == 0)
+    return FALSE;
+
+  return TRUE;
+}
+
 static void
 msg_drive_service_init (__attribute__ ((unused)) MsgDriveService *self)
 {
@@ -105,6 +183,7 @@ msg_drive_service_get_drives (MsgDriveService  *self,
       drive_object = json_array_get_object_element (array, index);
       drive = msg_drive_new_from_json (drive_object, &local_error);
       if (drive) {
+        self->type = msg_drive_get_drive_type (drive);
         list = g_list_append (list, drive);
       } else {
         g_warning ("Could not parse drive object: %s", local_error->message);
@@ -144,7 +223,7 @@ msg_drive_service_get_root (MsgDriveService  *self,
   if (!msg_service_refresh_authorization (MSG_SERVICE (self), cancellable, error))
     return NULL;
 
-  url = g_strconcat (MSG_API_ENDPOINT, "/drives/", msg_drive_get_id (drive), "/root", NULL);
+  url = g_strconcat (MSG_API_ENDPOINT, "/drives/", msg_drive_get_id (drive), "/root?select=id,remoteItem,file,folder,parentReference,name,createdBy,lastModifiedBy,createdDateTime,lastModifiedDateTime,size", NULL);
   message = msg_service_build_message (MSG_SERVICE (self), "GET", url, NULL, FALSE);
   parser = msg_service_send_and_parse_response (MSG_SERVICE (self), message, &root_object, cancellable, error);
   if (!parser)
@@ -171,6 +250,8 @@ msg_drive_service_download_item (MsgDriveService  *self,
                                  GError          **error)
 {
   g_autofree char *url = NULL;
+  const char *drive_id = NULL;
+  const char *id = NULL;
 
   if (!MSG_IS_DRIVE_ITEM_FILE (item)) {
     g_warning ("Download only allowed for files");
@@ -180,11 +261,19 @@ msg_drive_service_download_item (MsgDriveService  *self,
   if (!msg_service_refresh_authorization (MSG_SERVICE (self), cancellable, error))
     return NULL;
 
+  if (!msg_drive_item_is_shared (item)) {
+    drive_id = msg_drive_item_get_drive_id (item);
+    id = msg_drive_item_get_id (item);
+  } else {
+    drive_id = msg_drive_item_get_remote_drive_id (item);
+    id = msg_drive_item_get_remote_id (item);
+  }
+
   url = g_strconcat (MSG_API_ENDPOINT,
                      "/drives/",
-                     msg_drive_item_get_drive_id (item),
+                     drive_id,
                      "/items/",
-                     msg_drive_item_get_id (item),
+                     id,
                      "/content",
                      NULL);
 
@@ -217,23 +306,38 @@ msg_drive_service_list_children (MsgDriveService  *self,
   g_autolist (MsgDriveItem) children = NULL;
   g_autoptr (GBytes) response = NULL;
   g_autoptr (JsonParser) parser = NULL;
+  gboolean add_prefer_header = self->type == MSG_DRIVE_TYPE_BUSINESS;
+  const char *drive_id = NULL;
+  const char *id = NULL;
 
   if (!msg_service_refresh_authorization (MSG_SERVICE (self), cancellable, error))
     return NULL;
 
+  if (!msg_drive_item_is_shared (item)) {
+    drive_id = msg_drive_item_get_drive_id (item);
+    id = msg_drive_item_get_id (item);
+  } else {
+    drive_id = msg_drive_item_get_remote_drive_id (item);
+    id = msg_drive_item_get_remote_id (item);
+  }
+
   url = g_strconcat (MSG_API_ENDPOINT,
                      "/drives/",
-                     msg_drive_item_get_drive_id (item),
+                     drive_id,
                      "/items/",
-                     msg_drive_item_get_id (item),
+                     id,
                      "/children",
                      "?$expand=thumbnails",
+                     "&select=id,remoteItem,file,folder,parentReference,name,createdBy,lastModifiedBy,createdDateTime,lastModifiedDateTime,size",
                      NULL);
 
   do {
     g_autoptr (SoupMessage) message = NULL;
 
     message = msg_service_build_message (MSG_SERVICE (self), "GET", url, NULL /*msg_drive_item_get_etag (item)*/, FALSE);
+    if (add_prefer_header)
+      soup_message_headers_append (soup_message_get_request_headers (message), "Prefer", "Include-Feature=AddToOneDrive");
+
     parser = msg_service_send_and_parse_response (MSG_SERVICE (self), message, &root_object, cancellable, error);
     if (!parser)
       return NULL;
@@ -291,6 +395,14 @@ msg_drive_service_rename (MsgDriveService  *self,
   g_autoptr (GBytes) body = NULL;
   g_autoptr (JsonParser) parser = NULL;
 
+  if (!is_valid_name (new_name)) {
+    g_set_error (error,
+                 msg_error_quark (),
+                 MSG_ERROR_FAILED,
+                 "Invalid characters in name");
+    return NULL;
+  }
+
   if (!msg_service_refresh_authorization (MSG_SERVICE (self), cancellable, error))
     return NULL;
 
@@ -375,6 +487,14 @@ msg_drive_service_create_folder (MsgDriveService  *self,
   if (!msg_service_refresh_authorization (MSG_SERVICE (self), cancellable, error))
     return NULL;
 
+  if (!is_valid_name (name)) {
+    g_set_error (error,
+                 msg_error_quark (),
+                 MSG_ERROR_FAILED,
+                 "Invalid characters in name");
+    return NULL;
+  }
+
   url = g_strconcat (MSG_API_ENDPOINT,
                      "/drives/",
                      msg_drive_item_get_drive_id (parent),
@@ -537,7 +657,7 @@ msg_drive_service_update_finish (MsgDriveService  *self,
                                  GCancellable     *cancellable,
                                  GError          **error)
 {
-  g_autoptr (GBytes) bytes;
+  g_autoptr (GBytes) bytes = NULL;
   g_autofree char *url = NULL;
   g_autoptr (SoupMessage) message = NULL;
   JsonObject *root_object = NULL;
@@ -596,6 +716,14 @@ msg_drive_service_add_item_to_folder (MsgDriveService  *self,
   g_autoptr (GBytes) body = NULL;
   g_autoptr (JsonParser) parser = NULL;
 
+  if (!is_valid_name (msg_drive_item_get_name (item))) {
+    g_set_error (error,
+                 msg_error_quark (),
+                 MSG_ERROR_FAILED,
+                 "Invalid characters in name");
+    return NULL;
+  }
+
   if (!msg_service_refresh_authorization (MSG_SERVICE (self), cancellable, error))
     return FALSE;
 
@@ -651,6 +779,7 @@ msg_drive_service_get_shared_with_me (MsgDriveService  *self,
 
   url = g_strconcat (MSG_API_ENDPOINT,
                      "/me/drive/sharedWithMe",
+                     "?select=id,remoteItem,file,folder,parentReference,name,createdBy,lastModifiedBy,createdDateTime,lastModifiedDateTime,size",
                      NULL);
   do {
     g_autoptr (SoupMessage) message = NULL;
