@@ -30,6 +30,28 @@ MsgUserService *msg_user_service_new (MsgAuthorizer *authorizer);
 
 MsgUser *
 msg_user_service_get_user (MsgUserService  *self,
-                           char            *name,
+                           const char      *name,
                            GCancellable    *cancellable,
                            GError         **error);
+
+GBytes *
+msg_user_service_get_photo (MsgUserService  *self,
+                            const char      *mail,
+                            GCancellable    *cancellable,
+                            GError         **error);
+
+GList *
+msg_user_service_get_contacts (MsgUserService  *self,
+                               GCancellable    *cancellable,
+                               GError         **error);
+
+GList *
+msg_user_service_find_users (MsgUserService  *self,
+                             const char      *name,
+                             GCancellable    *cancellable,
+                             GError         **error);
+
+GList *
+msg_user_service_get_contact_folders (MsgUserService  *self,
+                                      GCancellable    *cancellable,
+                                      GError         **error);
